/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 ****************************************************************************/
 
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#ifndef _INT0_H
#define _INT0_H

#define RPM_SIGNALS_PER_TURN 2

/* example:
 * SIGNALS_PER_TURN 2 (4cylinder 4stroke engine)
 *
 * turns in rpm  || time between signals in s
 *       1       || 30
 *      10       || 3
 *     100       || 0,3
 *     150       || 0,2
 *     500       || 0,06
 *    1000       || 0,03
 *    7000       || 0,004.285.714.29
 *   10000       || 0,003
 *
 * => below 150rpm the engine is most-likely off
 */
#define MAX_VALID_RPM_TIME 200


void int0_init();

void int0_enable();

void int0_disable();

extern volatile uint16_t rpm;

// int0 Interrupt Routine
#if defined (__AVR_ATtiny84__)
ISR(EXT_INT0_vect);
#else
ISR(INT0_vect);
#endif

#endif
