/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 *                                                                          *
 *   Credits to:                                                            *
 *   Christian "quadratqualle" Lutze for code basics                        *
 *   Dario Carluccio for TID-Communication Description                      *
 ****************************************************************************/

#include "opel_tid.h"

void static inline TID_start() {
    sbi(TID_DDR, TID_MRQ);
    cbi(TID_PORT, TID_MRQ);
    _delay_us(1);
    while (TID_PIN & (1 << TID_SDA));
    _delay_us(10 WAIT_FACTOR);
    sbi(TID_PORT, TID_MRQ);
    while (!(TID_PIN & (1 << TID_SDA)));
    _delay_us(150 WAIT_FACTOR); //T4
    sbi(TID_DDR, TID_SDA);
    cbi(TID_PORT, TID_SDA);
    _delay_us(150 WAIT_FACTOR); //T6
    sbi(TID_DDR, TID_SCL);
    cbi(TID_PORT, TID_SCL);
}

void TID_send_byte(unsigned char daten) {
    daten <<= 1; // since last bit is parity, shift the data by one
    daten |= !parity_even_bit(daten); // and add parity

    //daten
    unsigned char a = 0b10000000;
    for (unsigned i = 0; i < 8; i++) {
        _delay_us(120 WAIT_FACTOR);
        sbi(TID_DDR, TID_SDA);
        if (a & daten) {
            //1
            sbi(TID_PORT, TID_SDA);
        } else {
            //0
            cbi(TID_PORT, TID_SDA);
        }
        _delay_us(5 WAIT_FACTOR);
        sbi(TID_PORT, TID_SCL);
        while (!(TID_PIN & (1 << TID_SCL)));
        _delay_us(120 WAIT_FACTOR);
        cbi(TID_PORT, TID_SCL);
        a >>= 1;
    }
    _delay_us(120 WAIT_FACTOR);
    cbi(TID_DDR, TID_SDA);
    cbi(TID_DDR, TID_SCL);
    while (!(TID_PIN & (1 << TID_SCL)));
    if (!(TID_PIN & (1 << TID_SDA))) {
        _delay_us(50 WAIT_FACTOR);
        cbi(TID_PORT, TID_SCL);
        sbi(TID_DDR, TID_SCL);
    }
}

void static inline TID_end() {
    sbi(TID_DDR, TID_SDA);
    cbi(TID_PORT, TID_SDA);
    _delay_us(100 WAIT_FACTOR);
    cbi(TID_DDR, TID_MRQ);
    _delay_us(100 WAIT_FACTOR);
    cbi(TID_DDR, TID_SCL);
    _delay_us(100 WAIT_FACTOR);
    cbi(TID_DDR, TID_SDA);
    _delay_us(100 WAIT_FACTOR);
}

void TID_puts(char buffer[], unsigned char symbols[]) {
    TID_start(); // start of transfer
    TID_send_byte(TID_address);
    cbi(TID_PORT, TID_MRQ);
    TID_send_byte(symbols[0]);
    TID_send_byte(symbols[1]);
#if TID_TYPE == 10
    TID_send_byte(symbols[2]);
#endif

    for (int i = 0; i < TID_characters; i++) {
        TID_send_byte(buffer[i]);
    }
    TID_end();
}

void TID_decimal(int16_t number, char description[], unsigned char symbols[], int scale) {
    char conv_array[TID_characters];
    char out_array[TID_characters];

    itoa(number, conv_array, 10);
    uint8_t len = strlen(conv_array);

    for (int8_t i = TID_characters - 1; i >= 0; i--) {
        if (len - (TID_characters - i) >= 0) {
            out_array[i] = conv_array[len - (TID_characters - i)];
        } else {
            out_array[i] = 0;
        }
    }

    // when scaled 0,0 looks nicer than ,0
    if (scale && len == 1) {
        out_array[TID_characters - 1 - len] = '0';
    }

    int8_t i = 0;
    while (description[i] != 0) {
        out_array[i] = description[i];
        i++;
    }

    if (scale) {
        symbols[0] |= TS_COMMA;
    } else {
        symbols[0] &= ~(TS_COMMA);
    }

    TID_puts(out_array, symbols);
}

void TID_scrolltext(char text[], unsigned char symbols[], uint8_t pgm, void (*functionP)(void)) {
    uint8_t len = 0;
    if (pgm) {
        while (pgm_read_byte(text + len++) != '\0');
    } else {
        while (text[len++] != '\0');
    }
    len--; // to be like strlen

    static char buffer[TID_characters + 1]; // all characters + \0

    for (uint8_t i = 0; i < len + TID_characters + 1; i++) {
        for (uint8_t pos = 0; pos < TID_characters; pos++) {
            buffer[pos] = (pos + i < TID_characters || pos + i - TID_characters > len - 1) ? ' ' : (pgm ? pgm_read_byte(text + pos + i - TID_characters) : text[pos + i - TID_characters]);
        }
        buffer[TID_characters] = '\0'; // actually not needed here but to be compliant with printf and co...

        // keep the delay between calls to <functionP>
        for (uint8_t x = 0; x < 3; x++) {
            (*functionP)();
            TID_puts(buffer, symbols);
            _delay_ms(SCROLLTEXTDELAY);
        }
    }
}

void TID_symbol_bargraph(unsigned char symbols[], int16_t min, int16_t max, int16_t value) {
    int16_t onepercent = (int16_t)((max - min) / 100);

    int16_t percent = value / onepercent;

    // save coma
    symbols[0] = (symbols[0] & TS_COMMA);
    symbols[1] = 0;

#if TID_TYPE == 10
    symbols[2] = 0;
    //TODO: need image from an 10char TID :)
#else
    if (percent > 11) {
        symbols[1] |= TS_DOLBYB | TS_DOLBYC;
    }
    if (percent > 22) {
        symbols[0] |= TS_AS;
    }
    if (percent > 33) {
        symbols[1] |= TS_CR;
    }
    if (percent > 44) {
        symbols[0] |= TS_STEREO;
    }
    if (percent > 55) {
        symbols[1] |= TS_CD;
    }
    if (percent > 66) {
        symbols[0] |= TS_RDS;
    }
    if (percent > 77) {
        symbols[1] |= TS_CPS;
    }
    if (percent > 88) {
        symbols[0] |= TS_TP | TS_TPBRACKETS;
    }
#endif
}
