/****************************************************************************
 *   Copyright (C) 2010-2011 by Claas Anders "CaScAdE" Rathje               *
 *   ClaasR@thje.net                                                        *
 *   Project-URL: http://www.mylifesucks.de/oss/c-tid/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 *                                                                          *
 ****************************************************************************/

#include "timer1.h"
#include "timer0.h"
#include "icp.h"
#include "int0.h"

// 16 bit timer
volatile uint16_t uptime = 0;

void timer1_init() {
#if defined (__AVR_ATtiny84__)		
    // CTC Mode
    TCCR1B = (1 << WGM12);

    // Prescale 8
    TCCR1B &= ~(1 << CS10);
    TCCR1B |= (1 << CS11);
    TCCR1B &= ~(1 << CS12);

    // Compare every 100ms at 8MHz
    OCR1A = 10000;

    // Compare Interrupt enable
    TIMSK1 |= (1 << OCIE1A);

#else 
    // CTC Mode
    TCCR1B = (1 << WGM12);

    // Prescale 8
    TCCR1B &= ~(1 << CS10);
    TCCR1B |= (1 << CS11);
    TCCR1B &= ~(1 << CS12);

    // Compare every 100ms at 16MHz
    OCR1A = 20000;

    // Compare Interrupt enable
    TIMSK |= (1 << OCIE1A);
#endif
}




// Timer0 Compare Interrupt Routine
#if defined (__AVR_ATtiny84__)

ISR(TIM1_COMPA_vect) {
#else

ISR(TIMER1_COMPA_vect) {
#endif

    static uint16_t ticks = 0;
    ticks++;

    if (ticks > 10) { // 1s?
        uptime++;
        ticks = 0;
    }
}
